import nugen as nn
from nudisxs.disxs import *
import time as time
import numpy as np
N =100
nun = nn.NuGen()
tt1 = np.zeros(N)
tt2 = np.zeros(N)
for i in range(N):
    tt1[i] = time.time()
    nun.get_event_fix_en()
    tt1[i] = time.time()-tt1[i]
    tt2[i] = time.time()
    nun.get_event()
    tt2[i] = time.time()-tt2[i]

print(tt1.mean())
print(tt2.mean())




import matplotlib
#from nudisxs.disxs import *
from pdg_constants import *
import matplotlib.pyplot as plt
import Earth as Earth
import Flux as flux
import cross as xs
import Global as g
import FinVtx as fin
import numpy as np
import time as time
import zf as zfactor

mag =  lambda x: np.sqrt(np.sum(x**2))

class NuPropagator:
    def __init__(self,flavor):
        print('Propogator is ready')
        self.flux = flux.Flux('KM','flux_data/Numu_H3a+KM_E2.DAT',3)
        self.i = 0
        self.xs_p=cs.Cross_section(model='CT10nlo',pdg=flavor,n_tt=2212)
        self.xs_n=cs.Cross_section(model='CT10nlo',pdg=flavor,n_tt=2112)

    def set_final_vertex(self,fvertex):
        self.fvertex = fvertex

    def set_event_number(self,i):
        self.i = i
    
    def prepare_propagation(self,i):
        self.earth=Earth.Earth(self.r,'PREM')
        self.spectra = self.flux.get_spectra_points()
        self.zfactor=zfactor.zf()
        self.set_event_number(i)
        self.energy = self.spectra[0][self.i]
        phi = np.random.random()
        cos = self.specta[1][self.i]
        sin = (1-cos**2)**0.5
        self.n = np.array([np.cos(phi)*sin,np.sin(phi)*sin,cos])
        scalar = np.sum(self.fvertex*self.n)
        self.ivertex  = self.fvertex-self.n(scalar+np.sqrt(scalar**2 - np.sum(self.vertex**2)+g.R_E**2))

    
    def probability(self,v1,v2,flavor,energy,N):            
        dep=self.earth.column_depth_vegas(v1,v2,N)
        self.depth=dep
        self.flux = self.spectra[2][self.i] 
        self.weight = self.spectra[3][self.i] 
        self.zf=self.zfactor.f(g.g*dep/g.cm**2,energy)
        prob = np.exp(-(self.Z*self.xs_p.tot(energy)+self.N*self.xs_m.tot(energy))
        prob = prob/(self.Z+self.N)*g.N_A*self.depth/g.g)
        prob_zf=np.exp(-(self.Z*self.xs_p.tot(energy)+self.N*self.xs_n.tot(energy))
        prob_zf = prob_zf/(self.Z+self.N)*(1-self.zf)*g.N_A*self.depth/g.g)
        return prob_zf,self.energy, self.cos, self.flux,self.weight

    def get_dragging(self,self.i,):
        self.prepare_propagation(self.i)
        info = self.probability(self.ivertex,self.fvertex,self.flavor)
        self.set_event_number(self.i+1)
        return info



import Earth as E
import numpy as np
import Global as g
import matplotlib.pyplot as plt
import FinVtx as fin
N = 100
epsrel = 10**(-3)
e=E.Earth(1,'PREM')
v1 = fin.FinVtx(0,0,g.R_E)
v2 = fin.FinVtx(0,0,-g.R_E)
v11 = np.array([0,0,g.R_E])
v22 = np.array([0,0,-g.R_E])
tt1 = np.zeros(N)
tt2 = np.zeros(N)
tt3 = np.zeros(N)
res1 = np.zeros(N)
res2 = np.zeros(N)
res3 = np.zeros(N)
res4 = np.zeros(N)
cc = np.zeros(N)
res_th = 1.09*10**10
for i in range(N):
    res1[i],tt1[i] = e.column_depth(v1,v2,1000)

for i in range(N):
    res2[i],cc[i],tt2[i] = e.column_depth_quad(v11,v22,10**(-3))

for i in range(N):
    res3[i],cc[i],tt3[i] = e.column_depth_vegas(v11,v22,10**(-3))
print('time',tt1.mean(),tt2.mean(),tt3.mean())
print('reldif',res1.mean()/res_th-1,res2.mean()/res_th-1,res3.mean()/res_th-1)
print(res2/res_th-1)
fig=plt.figure(figsize = (18,12))
plt.hist(res1/res_th-1,1000,range = [0.002,0.005],label='rect, time = '+str(tt1.mean())+r'$\pm$'+str(tt1.std()))
plt.hist(res2/res_th-1,1000,range = [0.002,0.005],label='quad, time = '+str(tt2.mean())+r'$\pm$'+str(tt2.std()))
plt.hist(res3/res_th-1,1000,range = [0.002,0.005],label='vegas, time = '+str(tt3.mean())+r'$\pm$'+str(tt3.std()))
plt.title('Integral difference')
plt.xlabel(r'density, $g/cm^3$')
plt.xlim(0,0.005)
plt.legend()
plt.grid(True)
plt.show()

fig.savefig('earth_test.png')




from types import SimpleNamespace
opts = SimpleNamespace()
opts.neutrinoPrimary_energy_mode = 'random'
opts.neutrinoPrimary_direction_mode = 'random'
opts.neutrinoPrimary_target_mode = 'random'
opts.neutrinoPrimary_current_mode = 'random'
opts.neutrinoPrimary_pdgid = 14
opts.neutrinoPrimary_energy_GeV = 10**2 
opts.neutrinoPrimary_direction = [0,0,1]
opts.neutrinoPrimary_pdf_model = 'CT10nlo'
opts.neutrinoPrimary_target = 2212

import nupropagator.nugen as  nugen
import numpy as np



nu = nugen.NuGen()
print('start')
nu.get_event(opts)
print('end')



import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from scipy.interpolate import interp1d
from scipy.interpolate import interp2d
import math as m
import Global as g


class Flux:
    def __init__(self, model_name, file, dim):
        df = pd.read_csv(file,header=None, sep ='\t')
        self.energy = []
        self.Ncos=11
        if dim == 2:
            self.Ncos=1
            
        self.NPoint=len(df)
        self.cos_theta = [1-float(j)/(self.Ncos-1.) for j in range(0,self.Ncos,1)]
        self.flux = np.zeros((self.Ncos,self.NPoint))
        self.model_name = model_name
        self.dim=dim
        self.energy = df[0]
        for i in range(self.Ncos):
            self.flux[i]=np.array(df[i+1])/np.array(df[0])/np.array(df[0])
        return
    def flux_x(self,E):
        a=0
        for i in range(self.Ncos):
            a+=2*np.pi*self.flux[i]*0.1
        f = interp1d(self.energy, a, kind='cubic')
        return f(E)
    def get_energy_flux_points_spl(self,cos_theta,N): #it is 1D interpolation for energy (cos is constant. it is one from 11 values)
        f = interp1d(self.energy, self.flux[int(cos_theta*10)], kind='cubic')
        z=[10*10.**((7.*i)/(N)) for i in range(0,N+1,1)]
        return z,f(z)
    
    def get_energy_flux_points_lin(self,cos_theta,N): #it is 1D interpolation for energy (cos is constant. it is one from 11 values)
        f = interp1d(self.energy, self.flux[int(cos_theta*10)], kind='linear')
        z=[10*10.**((7.*i)/(N)) for i in range(0,N+1,1)]
        return z,f(z)
    
    def get_energy_flux_points_spl1(self,cos_theta,z=[]): #it is 1D interpolation for energy (cos is constant. it is one from 11 values)
        f = interp1d(self.energy, self.flux[int(cos_theta*10)], kind='cubic')
        return f(z)
    
    def get_theta_flux_points_spl1(self,e,z=[]): #it is 1D interpolation for energy (cos is constant. it is one from 11 values)
        f = interp1d(self.cos_theta, self.flux[:,int(10*(m.log10(e)-1))], kind='cubic')
        return f(z)
          
    def get_3D_data_points(self):
        return self.flux
    
    def get_3D_splain_points1(self,x_low=10.,x_up=10**8,Nxp=100,y_low=0,y_up=1,Nyp=100):
        I2 = interp2d(self.energy, self.cos_theta, self.flux, kind='cubic')
        z1=np.array([x_low*10**(m.log10(x_up/x_low)*i/Nxp) for i in range(0,Nxp+1,1)])
        z2=np.array([(y_up-y_low)*i/Nyp+y_low for i in range(0, Nyp+1,1)])
        return z1,z2,I2(z1,z2)
        
    def get_3D_splain_points(self,z1,z2):
        I2 = interp2d(self.energy, self.cos_theta, self.flux, kind='cubic')
        return I2(z1,z2)
    
    def get_3D_linear_points1(self,z1,z2):
        I2 = interp2d(self.energy, self.cos_theta, self.flux, kind='linear')
        return I2(z1,z2)
    



















import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from nudisxs.disxs import *
from scipy.interpolate import interp1d
from scipy.interpolate import interp2d
import math as m
import Global as g
import vegas 

class Flux:
    def __init__(self, model_name, file, dim):
        df = pd.read_csv(file,header=None, sep ='\t')
        self.energy = []
        self.Ncos=11
        if dim == 2:
            self.Ncos=1
            
        self.NPoint=len(df)
        self.cos_theta = [1-float(j)/(self.Ncos-1.) for j in range(0,self.Ncos,1)]
        self.flux = np.zeros((self.Ncos,self.NPoint))
        self.model_name = model_name
        self.dim=dim
        self.energy = np.array(df[0])
        for i in range(self.Ncos):
            self.flux[i]=np.array(df[i+1])/np.array(df[0])/np.array(df[0])
        return
    def flux_x(self,E):
        a=0
        for i in range(self.Ncos):
            a+=2*np.pi*self.flux[i]*0.1
        f = interp1d(self.energy, a, kind='cubic')
        return f(E)
    def get_energy_flux_points_spl(self,cos_theta,N): #it is 1D interpolation for energy (cos is constant. it is one from 11 values)
        f = interp1d(self.energy, self.flux[int(cos_theta*10)], kind='cubic')
        z=[10*10.**((7.*i)/(N)) for i in range(0,N+1,1)]
        return z,f(z)
    
    def get_energy_flux_points_lin(self,cos_theta,N): #it is 1D interpolation for energy (cos is constant. it is one from 11 values)
        f = interp1d(self.energy, self.flux[int(cos_theta*10)], kind='linear')
        z=[10*10.**((7.*i)/(N)) for i in range(0,N+1,1)]
        return z,f(z)
    
    def get_energy_flux_points_spl1(self,cos_theta,z=[]): #it is 1D interpolation for energy (cos is constant. it is one from 11 values)
        f = interp1d(self.energy, self.flux[int(cos_theta*10)], kind='cubic')
        return f(z)
    
    def get_theta_flux_points_spl1(self,e,z=[]): #it is 1D interpolation for energy (cos is constant. it is one from 11 values)
        f = interp1d(self.cos_theta, self.flux[:,int(10*(m.log10(e)-1))], kind='cubic')
        return f(z)
          
    def get_3D_data_points(self):
        return self.flux
    
    def get_3D_splain_points1(self,x_low=10.,x_up=10**8,Nxp=100,y_low=0,y_up=1,Nyp=100):
        I2 = interp2d(self.energy, self.cos_theta, self.flux, kind='cubic')
        z1=np.array([x_low*10**(m.log10(x_up/x_low)*i/Nxp) for i in range(0,Nxp+1,1)])
        z2=np.array([(y_up-y_low)*i/Nyp+y_low for i in range(0, Nyp+1,1)])
        return z1,z2,I2(z1,z2)
        
    def get_3D_splain_points(self,z1,z2):
        I2 = interp2d(self.energy, self.cos_theta, self.flux, kind='cubic')
        self.I2 = I2
        return I2(z1,z2)
    
    def get_3D_linear_points1(self,z1,z2):
        I2 = interp2d(self.energy, self.cos_theta, self.flux, kind='linear')
        return I2(z1,z2)
    
    #@vegas.batchintegrand
    def get_vegas_flux(self,xx):
        #z = self.I2(xx[:,0],xx[:,1])
        #z = xx[:,0]+xx[:,1]
        z = self.get_3D_splain_points(xx[0],xx[1])
        return np.array(z)
    
    def get_spectra_points(self):
        self.integrator = vegas.Integrator([[self.energy.min(),self.energy.max()], [-1,1]], nhcube_batch=2000, sync_ran=False)
        #self.integrator = vegas.Integrator([[self.energy.min(),self.energy.max()], [-1,1]])
        result = self.integrator(self.get_vegas_flux, nitn = 3,neval = 1000)
        print(result.summary)
        E = []
        cos = []
        weight = []
        for x,wgt in self.integrator.random():
            E.append(x[0])
            cos.append(x[1])
            weight.append(wgt)
        E = np.array(E)
        cos = np.array(cos)
        weight = np.array(weight)
        flux = np.zeros(len(E))
        flux = self.I2(E,cos)
        return E,cos,weight,flux


fff = Flux('KM','flux_data/Numu_H3a+KM_E2.DAT',3)
fff.get_3D_splain_points(fff.energy,fff.cos_theta)
a = fff.get_spectra_points()
print(a[0])









